'''
Started on: 2011.07.20
Last update: nil
Author: Kulverstukas && Factionwars

=================================================
Copyright 2011 Kulverstukas, Factionwars

    This file is part of D0xBase.

    D0xBase is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation version 3

    D0xBase is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with D0xBase.  If not, see http://www.gnu.org/licenses/
=================================================

Shouts: Evilzone.org
You can contact us at: kulverwars@evilzone.org
--------
Description:
  This module is part of "D0xBase" project.
  Any modification without knowing what you are doing will result
  in unwanted behavior and can stop working.
--------
'''

'''
======
Fonts:
urwid.font.Thin3x3Font
urwid.font.Thin4x3Font
urwid.font.HalfBlock5x4Font
urwid.font.HalfBlock6x5Font
urwid.font.HalfBlockHeavy6x5Font
urwid.font.Thin6x6Font
urwid.font.HalfBlock7x7Font
======
'''

import urwid, datetime, random, os, time, d0xengine

''' Front end starts here '''
#======================================
__version__ = '0.1'

def RandomColor():
    '''Pick a random color for the main menu text. Just for phun :D'''
    ListOfColors = ['dark red', 'dark green', 'brown', 'dark blue',
                    'dark magenta', 'dark cyan', 'light gray',
                    'dark gray', 'light red', 'light green', 'yellow',
                    'light blue', 'light magenta', 'light cyan', 'default']
    Color = ListOfColors[random.randint(0, 14)]
    return Color

#Screen colors for every element
Palette = [
           ('Field', 'dark green, bold', 'black'), # information fields, like Name, Age etc.
           ('Info', 'dark green', 'black'), # information in fields
           ('Bg', 'black', 'black'), # screen background
           ('InfoFooterText', 'white', 'dark blue'), # footer text
           ('InfoFooterHotkey', 'dark cyan, bold', 'dark blue'), # hotkeys in footer text
           ('InfoFooter', 'black', 'dark blue'),  # footer background
           ('InfoHeaderText', 'white, bold', 'dark blue'), # header text
           ('InfoHeader', 'black', 'dark blue'), # header background
           ('BigText', RandomColor(), 'black'), # main menu banner text
           ('GeneralInfo', 'brown', 'black'), # main menu text
           ('LastModifiedField', 'dark cyan, bold', 'black'), # Last modified:
           ('LastModifiedDate', 'dark cyan', 'black'), # info in Last modified:
           ('PopupMessageText', 'black', 'dark cyan'), # popup message text
           ('PopupMessageBg', 'black', 'dark cyan'), # popup message background
           ('SearchBoxHeaderText', 'light gray, bold', 'dark cyan'), # field names in the search box
           ('SearchBoxHeaderBg', 'black', 'dark cyan'), # field name background in the search box
           ('OnFocusBg', 'white', 'dark magenta') # background when a widget is focused
          ]
#======================================
#Different text for different screens
HeaderText = 'D0xBase version %s' % __version__

FooterViewText = ('InfoFooterText', ['Press ', ('InfoFooterHotkey', '"ESC"'), ' for main screen, ', ('InfoFooterHotkey', '"E"'), ' to edit, ', ('InfoFooterHotkey', '"A"'), ' to add, ', ('InfoFooterHotkey', '"D"'), ' to delete, ', ('InfoFooterHotkey', '"S"'), ' to search'])

FooterAddingText = ('InfoFooterText', ['Press ', ('InfoFooterHotkey', '"ESC"'), ' to Cancel, ', ('InfoFooterHotkey', '"F2"'), ' to add'])

FooterMainText = ('InfoFooterText', ['Press ', ('InfoFooterHotkey', '"ESC"'), ' to Quit, ', ('InfoFooterHotkey', '"A"'), ' to add, ', ('InfoFooterHotkey', '"S"'), ' to search'])

FooterSearchText = ('InfoFooterText', ['Press ', ('InfoFooterHotkey', '"ESC"'), ' for main screen'])

SearchScreenFooterText = ('InfoFooterText', ['Press ', ('InfoFooterHotkey', '"ESC"'), ' for main screen, ', ('InfoFooterHotkey', '"A"'), ' to add, ', ('InfoFooterHotkey', '"S"'), ' to search again'])
#======================================
#Global vars
EditFieldsList = urwid.ListBox
Information = []
INPUT = []
RadioButtons = []
NumberOfPeople = 0
CameToEdit = False
SearchField = urwid.Edit()
IntEdit = urwid.IntEdit()
#======================================
class FrontEnd:
    '''The graphical front-end for D0xBase engine'''
#======================================
    def CalculateAge(self, Date):
        '''Calculates the age and days until next birthday from the given birth date'''
        try:
            for I in './- ':
                if (Date.find(I) != -1):
                    Date = Date.split(I)
                    break;
            BirthDate = datetime.date(int(Date[0]), int(Date[1]), int(Date[2]))
            Today = datetime.date.today()
    
            if (Today.month > BirthDate.month):
                NextYear = datetime.date(Today.year + 1, BirthDate.month, BirthDate.day)
            elif (Today.month < BirthDate.month):
                NextYear = datetime.date(Today.year, Today.month + (BirthDate.month - Today.month), BirthDate.day)
            elif (Today.month == BirthDate.month):
                if (Today.day > BirthDate.day):
                    NextYear = datetime.date(Today.year + 1, BirthDate.month, BirthDate.day)
                elif (Today.day < BirthDate.day):
                    NextYear = datetime.date(Today.year, BirthDate.month, Today.day + (BirthDate.day - Today.day))
                elif (Today.day == BirthDate.day):
                    NextYear = 0

            Age = Today.year - BirthDate.year

            if NextYear == 0: #if today is the birthday
                return '%d, days until %d: %d' % (Age, Age+1, 0)
            else:
                DaysLeft = NextYear - Today
                return '%d, days until %d: %d' % (Age, Age+1, DaysLeft.days)
        except:
            return 'Wrong date format'
#======================================
    def CalculateDays(self, Date):
        '''Calculates number of days the record wasn't modified'''
        try:
            Date = Date.split('.')
            Date = datetime.date(int(Date[0]), int(Date[1]), int(Date[2]))
            Today = datetime.date.today()
            DaysSince = Today - Date
            return '%d.%d.%d, days since then: %d' % (Date.year, Date.month, Date.day, DaysSince.days)
        except:
             return 'Wrong date format was written'
#======================================
    def CheckDate(self, Str):
        '''Checks a date if it is in a valid format'''
        try:
            if (time.strptime(Str, '%Y.%m.%d')): return True
        except:
            return False
#======================================
    def ReplaceBlanks(self, Input):
        '''Replaces blanks with a string'''
        try:
            NewDiction = []
            for Item in Input:
                if (Item == '') or (Item == None):
                    Item = 'Not filled'
                NewDiction.append(Item)
            return NewDiction
        except:
            return Input
#======================================
#This makes the output look cool and displays it
#Input needs to be passed formatted in a specific way
#Such as: String_String_String_...
#Fields must be passed in order as defined in "db_fields"

    def FormatOutput(self, Input):
        '''Format the output with information and return a listbox of epicness'''
        Age = FrontEnd().CalculateAge(INPUT[5]) #can has age?
        #Input = FrontEnd().ReplaceBlanks(Input)
        if (len(Input) != 17): #oh shii-- something is missing!
            return urwid.ListBox(urwid.SimpleListWalker([urwid.Text(('LastModifiedField', 'Error in database output. Try restarting the program.'))]))

        Output = [
           urwid.Columns([urwid.Padding(urwid.Text([
               ('LastModifiedField', 'Last modified: '), ('LastModifiedDate', self.CalculateDays(Input[16])+'\n\n')
                                                   ], align='center'), left=10)], 1),
           urwid.Columns([urwid.Text([('Field', 'Name: '), ('Info', Input[1])]), 
                          urwid.Text([('Field', 'Last name: '), ('Info', Input[2])])
                         ], 2),
           urwid.Columns([urwid.Text([('Field', 'Nickname: '), ('Info', Input[3])]),
                          urwid.Text([('Field', 'Gender: '), ('Info', Input[4])])
                         ], 2),
           urwid.Columns([urwid.Text([('Field', 'Date of birth: '), ('Info', Input[5])]),
                          urwid.Text([('Field', 'Place of birth: '), ('Info', Input[6])])
                         ], 2),
           urwid.Columns([urwid.Text([('Field', 'Date of death: '), ('Info', Input[7])]),
                          urwid.Text([('Field', 'Place of death: '), ('Info', Input[8])])
                         ], 2),
           urwid.Columns([urwid.Text([('Field', 'Age: '), ('Info', Age.ljust(31))]),
                          urwid.Text([('Field', 'Location: '), ('Info', Input[9])])
                         ], 2),
           urwid.Columns([urwid.Text([('Field', 'Email: '), ('Info', Input[10])]),
                          urwid.Text([('Field', 'Education: '), ('Info', Input[11])])
                         ], 2),
           urwid.Columns([urwid.Text([('Field', 'Occupation: '), ('Info', Input[12])]),
                          urwid.Text([('Field', 'Workplace: '), ('Info', Input[13])])
                         ], 2),
           urwid.Columns([urwid.Text([('Field', 'Contact:\n'), ('Info', Input[14])]),
                          urwid.Text([('Field', 'Notes:\n'), ('Info', Input[15])])
                         ], 2)
                 ]
        return urwid.ListBox(urwid.SimpleListWalker(Output))
#======================================
    def ShowInfo(self):
        '''Part of the Search function. Search calls this function to display found data'''
       
        #Footer
        Footer = urwid.AttrMap(urwid.Text(FooterViewText, align='center'), 'InfoFooter')

        #Header
        Header = urwid.AttrWrap(urwid.Text(('InfoHeaderText', '%s %s Viewing' % (HeaderText, '/')), align='center'), 'InfoHeader')       

        PaddedBody = urwid.Overlay(FrontEnd().FormatOutput(INPUT), urwid.SolidFill(' '), 'center', 80, 'middle', 20)
        HeadBodyFootFrame = urwid.Frame(body=PaddedBody, header=Header, footer=Footer)

        #Background
        Bkg = urwid.AttrWrap(HeadBodyFootFrame, 'Bg')
        #Bkg = urwid.LineBox(Bkg)

        #Deletion popup
        Delete = urwid.Text(('PopupMessageText', 'Are you sure you want to delete it?\n[Y/N]'), align='center')
        Delete = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(Delete, title='|** Confirm **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        return  Bkg, Delete
#======================================
    def ShowAdd(self):
        '''This is the adding screen. Lets you add people to the database'''
        global EditFieldsList

        #Footer
        Footer = urwid.AttrMap(urwid.Text(FooterAddingText, align='center'), 'InfoFooter')

        #Header
        Header = urwid.AttrMap(urwid.Text(('InfoHeaderText', '%s %s Adding' % (HeaderText, '/')), align='center'), 'InfoHeader')

        EditFields = urwid.SimpleListWalker([
                       urwid.AttrWrap(urwid.Edit(('Field', 'Name............: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Last name.......: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Nickname........: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Gender..........: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Date of birth...: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Place of birth..: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Date of death...: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Place of death..: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Location........: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Email address...: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Education.......: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Occupation......: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Workplace.......: ')), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Contact.........: '), multiline=True), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Notes...........: '), multiline=True), 'Info', 'OnFocusBg'),
                       urwid.Divider()
                                             ])
        Explanation = urwid.SimpleListWalker([
                        urwid.Padding(urwid.Text(('GeneralInfo', '\n Tips and tricks filling this form:\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  1. While filling the form you can paste '\
                                                   'text to it using the mouse. The "Name" '\
                                                   'field MUST be filled to add a record to the database\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  2. Move up and down using  arrow keys. '\
                                                   'You can also use "Page up" and "Page down" '\
                                                   'to get to the first/last line.\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  3. When filling the "Contact" field or the "Notes" '\
                                                   'field, to move text to a new line, press "ENTER".\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  4. When you are done filling the form, press "F2" '\
                                                   'to save everything to the database. Not everything must '\
                                                   'be filled in. For those fields that there is no info '\
                                                   'you can leave them empty and edit the record later on.\n')), left=2)
                                              ])
        EditFieldsList = urwid.ListBox(EditFields)
        Explanation = urwid.ListBox(Explanation)

        vline = urwid.AttrWrap(urwid.SolidFill('|'), 'Info')

        BodyWithInfo = urwid.Columns([('fixed', 40, Explanation),('fixed', 1, vline), EditFieldsList], dividechars=3, focus_column=2)
        HeadBodyFootFrame = urwid.Frame(footer=Footer, header=Header, body=BodyWithInfo)

        #Background
        Bkg = urwid.AttrWrap(HeadBodyFootFrame, 'Bg')
        #Bkg = urwid.LineBox(Bkg)

        #Empty Name field dialog
        Empty = urwid.Text(('PopupMessageText', 'Cannot add to the database.\n"Name" field must be filled! Letters must be used.\n\nPress "ESC" to return'), align='center')
        Empty = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(Empty, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')
        #Empty = urwid.LineBox(Empty)

        #Wrong date popup
        WrongDate = urwid.Text(('PopupMessageText', 'Cannot add to the database.\nWrong date format!\nDate must be like YEAR.MONTH.DAY\n\nPress "ESC" to return'), align='center')
        WrongDate = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(WrongDate, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Message to show when person is added
        Added = urwid.Text(('PopupMessageText', 'Successfully added to the database!\n\nPress "ESC" to return'), align='center')
        Added = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(Added, title='|** Success **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Message to show when canceling adding
        ReallyCancel = urwid.Text(('PopupMessageText', 'Are you sure you want to cancel?\n[Y/N]'), align='center')
        ReallyCancel = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(ReallyCancel, title='|** Confirm **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Message to show when adding a record
        ReallyAdd = urwid.Text(('PopupMessageText', 'Are you sure you want to add?\n[Y/N]'), align='center')
        ReallyAdd = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(ReallyAdd, title='|** Confirm **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        return Bkg, Empty, WrongDate, Added, ReallyCancel, ReallyAdd
#======================================
    def ShowEditor(self):
        '''This is the editing screen. Lets you edit people in the database'''
        global EditFieldsList
        #Footer
        Footer = urwid.AttrMap(urwid.Text(FooterAddingText, align='center'), 'InfoFooter')

        #Header
        Header = urwid.AttrMap(urwid.Text(('InfoHeaderText', '%s %s Editing' % (HeaderText, '/')), align='center'), 'InfoHeader')

        EditFields = urwid.SimpleListWalker([
                       urwid.AttrWrap(urwid.Edit(('Field', 'Name............: '), INPUT[1]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Last name.......: '), INPUT[2]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Nickname........: '), INPUT[3]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Gender..........: '), INPUT[4]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Date of birth...: '), INPUT[5]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Place of birth..: '), INPUT[6]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Date of death...: '), INPUT[7]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Place of death..: '), INPUT[8]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Location........: '), INPUT[9]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Email address...: '), INPUT[10]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Education.......: '), INPUT[11]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Occupation......: '), INPUT[12]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Workplace.......: '), INPUT[13]), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Contact.........: '), INPUT[14], multiline=True), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                       urwid.AttrWrap(urwid.Edit(('Field', 'Notes...........: '), INPUT[15], multiline=True), 'Info', 'OnFocusBg'),
                       urwid.Divider(),
                                             ])
        Explanation = urwid.SimpleListWalker([
                        urwid.Padding(urwid.Text(('GeneralInfo', '\n Tips and tricks filling this form:\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  1. While filling the form you can paste '\
                                                   'text to it using the mouse. The "Name" '\
                                                   'field MUST be filled to add a record to the database\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  2. Move up and down using  arrow keys. '\
                                                   'You can also use "Page up" and "Page down" '\
                                                   'to get to the first/last line.\n')), left=1),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  3. When filling the "Contact" field or the "Notes" '\
                                                   'field, to move text to a new line, press "ENTER".\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  4. When you are done filling the form, press "F2" '\
                                                   'to save everything to the database. Not everything must '\
                                                   'be filled in. For those fields that there is no info '\
                                                   'you can leave them empty and edit the record later on.\n')), left=2)
                                              ])
        EditFieldsList = urwid.ListBox(EditFields)
        Explanation = urwid.ListBox(Explanation)

        vline = urwid.AttrWrap(urwid.SolidFill('|'), 'Info')

        BodyWithInfo = urwid.Columns([('fixed', 40, Explanation),('fixed', 1, vline), EditFieldsList], dividechars=3, focus_column=2)
        HeadBodyFootFrame = urwid.Frame(footer=Footer, header=Header, body=BodyWithInfo)

        #Background
        Bkg = urwid.AttrWrap(HeadBodyFootFrame, 'Bg')
        #Bkg = urwid.LineBox(Bkg)

        #Empty Name field dialog
        Empty = urwid.Text(('PopupMessageText', 'Cannot add to the database.\n"Name" field must be filled!\n\nPress "ESC" to return'), align='center')
        Empty = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(Empty, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Wrong date popup
        WrongDate = urwid.Text(('PopupMessageText', 'Cannot add to the database.\nWrong date format!\nDate must be like YEAR.MONTH.DAY\n\nPress "ESC" to return'), align='center')
        WrongDate = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(WrongDate, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Message to show when person is added
        Added = urwid.Text(('PopupMessageText', 'Successfully added to the database!\n\nPress "ESC" to return'), align='center')
        Added = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(Added, title='|** Success **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Message to show when canceling
        ReallyCancel = urwid.Text(('PopupMessageText', 'Are you sure you want to cancel?\n[Y/N]'), align='center')
        ReallyCancel = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(ReallyCancel, title='|** Confirm **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Message to show when adding a record
        ReallyAdd = urwid.Text(('PopupMessageText', 'Are you sure you want to add?\n[Y/N]'), align='center')
        ReallyAdd = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(ReallyAdd, title='|** Confirm **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        return Bkg, Empty, WrongDate, Added, ReallyCancel, ReallyAdd
#======================================
    def ShowSearch(self):
        '''This is a search screen, to search people in the database'''
        global RadioButtons, SearchField
        #Footer
        Footer = urwid.AttrMap(urwid.Text(FooterSearchText, align='center'), 'InfoFooter')

        #Header
        Header = urwid.AttrMap(urwid.Text(('InfoHeaderText', '%s %s Searching' % (HeaderText, '/')), align='center'), 'InfoHeader')

        RadioGroup = []
        RadioButtons = [ 
                         urwid.AttrWrap(urwid.RadioButton(RadioGroup, 'ID', state=False), 'Info', 'OnFocusBg'),
                         urwid.AttrWrap(urwid.RadioButton(RadioGroup, 'Name', state=True), 'Info', 'OnFocusBg'),
                         urwid.AttrWrap(urwid.RadioButton(RadioGroup, 'Last name', state=False), 'Info', 'OnFocusBg'),
                         urwid.AttrWrap(urwid.RadioButton(RadioGroup, 'Nickname', state=False), 'Info', 'OnFocusBg')
                       ]
        SearchField = urwid.AttrWrap(urwid.Edit(('Field', 'Search: ')), 'Info', 'OnFocusBg')
                                             

        Explanation = urwid.SimpleListWalker([
                        urwid.Padding(urwid.Text(('GeneralInfo', '\n Tips and tricks for searching:\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  1. Searching by ID, you only need to enter '\
                                                                 'the ID of the person in the Database. '\
                                                                 'So, if you would enter "1337", you would get '\
                                                                 'a person whose ID is "1337"\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  2. Searching by "Name", "Last name" or "Nickname", '\
                                                                 'only enter one of the above to get a list '\
                                                                 'of records matching that criteria\n')), left=2),
                        urwid.Padding(urwid.Text(('GeneralInfo', '  3. To select how to search, use the arrow keys, '\
                                                                 'UP and DOWN, when the cursor is on a wanted box '\
                                                                 'press ENTER or SPACE to select it\n')), left=2),
                                              ])

        Bodeh = urwid.Pile([urwid.ListBox(urwid.SimpleListWalker([
                     urwid.Divider(),
                     SearchField,
                     urwid.Divider(),
                     urwid.AttrWrap(urwid.Text(('Field', 'Select by what to search:')), 'Bg'),
                     RadioButtons[0],
                     RadioButtons[1],
                     RadioButtons[2],
                     RadioButtons[3]]))
                           ])

        Explanation = urwid.ListBox(Explanation)

        BodyWithInfo = urwid.Columns([('fixed', 40, Explanation), ('fixed', 1, urwid.AttrWrap(urwid.SolidFill('|'), 'Info')), Bodeh], dividechars=3, focus_column=2)
        HeadBodyFootFrame = urwid.Frame(footer=Footer, header=Header, body=BodyWithInfo)

        #Background
        Bkg = urwid.AttrWrap(HeadBodyFootFrame, 'Bg')
        #Bkg = urwid.LineBox(Bkg)

        #Empty field popup
        Empty = urwid.Text(('PopupMessageText', 'Cannot proceed with the search.\nNothing given as search criteria!\n\nPress "ESC" to return'), align='center')
        Empty = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(Empty, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Search by ID received a letter. We are doomed!
        GotLetter = urwid.Text(('PopupMessageText', 'Cannot proceed with the search\nSearch by ID received a letter!\nMust only be numbers.\n\nPress "ESC to return"'), align='center')
        GotLetter = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(GotLetter, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        #Record does not exist
        NoRecord = urwid.Text(('PopupMessageText', 'Cannot proceed with the search\nInvilid number! Too few records maybe?\n\nPress "ESC to return"'), align='center')
        NoRecord = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(NoRecord, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        return Bkg, Empty, GotLetter, NoRecord
#======================================
    def ShowSearchResultScreen(self, ListOfResults):
        '''Search result screen'''
        global IntEdit

        ResultBoxColumns = [
                    urwid.AttrWrap(urwid.Text(('SearchBoxHeaderText', 'ID'), align='center'), 'SearchBoxHeaderBg'),
                    urwid.AttrWrap(urwid.Text(('SearchBoxHeaderText', 'Name'), align='center'), 'SearchBoxHeaderBg'),
                    urwid.AttrWrap(urwid.Text(('SearchBoxHeaderText', 'Last name'), align='center'), 'SearchBoxHeaderBg'),
                    urwid.AttrWrap(urwid.Text(('SearchBoxHeaderText', 'Birth date'), align='center'), 'SearchBoxHeaderBg')
                           ]

        #Let's put all the results in columns, k?
        ResTmp = []
        ResultToShow = []
        for List in ListOfResults:
            for Item in List:
                ResTmp.append(urwid.AttrWrap(urwid.Text(('Info', '%s' % Item), align='center'), 'Bg'))
            ResultToShow.append(urwid.Columns(ResTmp, 1))
            ResTmp = []

        IntEdit = urwid.AttrWrap(urwid.IntEdit(caption='        Enter ID: '), 'Field')
        #IntEditPadded = urwid.Padding(IntEdit)
        ResultBox = urwid.ListBox(urwid.SimpleListWalker([
                                        urwid.Divider(),
                                        IntEdit,
                                        urwid.Divider(),
                                        urwid.Columns(ResultBoxColumns, 1),
                                                         ]+ResultToShow))

        #Footer
        Footer = urwid.AttrMap(urwid.Text(SearchScreenFooterText, align='center'), 'InfoFooter')

        #Header
        Header = urwid.AttrMap(urwid.Text(('InfoHeaderText', '%s %s Search results %s Found records: %d' % (HeaderText, '/', '/', len(ResultToShow))), align='center'), 'InfoHeader')

        HeadBodyFootFrame = urwid.Frame(footer=Footer, header=Header, body=ResultBox)

        #Background
        Bkg = urwid.AttrWrap(HeadBodyFootFrame, 'Bg')

        #Record does not exist
        NoRecord = urwid.Text(('PopupMessageText', 'Cannot proceed with the search\nInvilid number! Too few records maybe?\n\nPress "ESC to return"'), align='center')
        NoRecord = urwid.AttrWrap(urwid.Overlay(urwid.LineBox(NoRecord, title='|** Error **|'), Bkg, 'center', 40, 'middle', None), 'PopupMessageBg')

        return Bkg, NoRecord
#======================================
    def ShowMain(self):
        '''The main screen'''
        #Footer
        Footer = urwid.AttrMap(urwid.Text(FooterMainText, align='center'), 'InfoFooter')

        #Header
        Header = urwid.AttrMap(urwid.Text(('InfoHeaderText', '%s %s Kulverstukas && Factionwars @ Evilzone.org' % (HeaderText, '/')), align='center'), 'InfoHeader')

        #Banner
        Banner = urwid.AttrWrap(urwid.Overlay(urwid.BigText('D0xBase %s' % __version__, urwid.font.HalfBlock7x7Font()), urwid.SolidFill(' '), 'center', None, 'top', None), 'BigText')

        #Message below the banner
        MainPageText = urwid.LineBox(urwid.AttrWrap(urwid.Overlay(urwid.LineBox(urwid.Padding(urwid.Text(('GeneralInfo', 'GNU General Public Licence\n\nThis file is part of D0xBase. D0xBase is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation version 3. D0xBase is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.\nYou should have received a copy of the GNU General Public License along with D0xBase. If not, see http://www.gnu.org/licenses/\n\nContact information\n\nIn case of any bugs, suggestions or just to say Thanks, please email us at kulverwars@evilzone.org ([Kulver]stukas+Faction[wars] :D)\nYou can get a copy of this application by going to http://newage.ql.lt/projects/python/d0xbase/'), align='center'), align='center', width='pack', left=3, right=3)), Banner, 'center', 150, 'bottom', None), 'GeneralInfo'))
        HeadBodyFootFrame = urwid.Frame(header=Header, body=MainPageText, footer=Footer)
        Bkg = urwid.AttrWrap(HeadBodyFootFrame, 'Bg')
        return Bkg
#======================================
    def CheckForChars(self, Item):
        '''Checks a string if it contains a letter'''
        for Char in 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ ':
            if Item.find(Char) != -1:
                return True
        return False
#======================================
    def GetResults(self, ListWithIDs):
        '''Receives a list of tuples with IDs and returns relevant data in 2D array'''
        IDs = []
        Result = []
        FinalResult = []
        for Item in ListWithIDs:
            IDs.append(Item[0]) #extract the IDs from tuples
        for Item in IDs:
            Persons = database.getperson(Item)
            Result.append(Persons[0]) # ID
            Result.append(Persons[1]) # first name
            Result.append(Persons[2]) # last name
            Result.append(Persons[5]) # birth date
            FinalResult.append(Result)
            Result = []
        return FinalResult
#======================================
    def CancelAdding(self, key):
        '''Adding and editing screen menu'''
        global EditFieldsList, Information, INPUT, CameToEdit, NumberOfPeople
        if (self.loop.widget == self.exit) or (self.loop.widget == self.WrongDate):
            if key == 'esc':
                self.loop.widget = self.MainScreen
                return
        elif (self.loop.widget == self.ConfirmCancel):
            if key in ('y', 'Y'):
                self.loop = urwid.MainLoop(self.ShowMain(), Palette, unhandled_input=self.MainScreenMenu)
                self.loop.run()
            elif key in ('n', 'N'):
                self.loop.widget = self.MainScreen
                return
        elif (self.loop.widget == self.ReallyAdd):
            #if a record is to be modified, "CameToEdit" will be true
            if key in ('y', 'Y'):
                if (CameToEdit == True):
                    Information.insert(0, INPUT[0])
                    database.editperson(Information)
                    CameToEdit = False
                    #Information = []  #"The girl was so ugly that I had to clear my cache" :D
                    self.loop.widget = self.Success
                    return
                else:
                    #Let's add that ish, finally!
                    database.addperson(Information)
                    Information = []  #"The girl was so ugly that I had to clear my cache" :D
                    NumberOfPeople += 1
                    self.loop.widget = self.Success
                    return
            elif key in ('n', 'N'):
                self.loop.widget = self.MainScreen
                return
        elif key == 'esc':
            if (self.loop.widget == self.Success):
                if (CameToEdit == True):
                    INPUT = Information
                    Information = []
                    CameToEdit = False
                    self.MainScreen, self.DeletionPopup = self.ShowInfo()
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.ViewScreenMenu)
                    self.loop.run()
                else:
                    Information = []
                    INPUT = database.getperson(NumberOfPeople)
                    self.MainScreen, self.DeletionPopup = self.ShowInfo()
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.ViewScreenMenu)
                    self.loop.run()
            else:
                self.loop.widget = self.ConfirmCancel
                return
        elif key == 'f2':
            #Get original place where the cursor was
            Hurr, Durr = EditFieldsList.get_focus()
            #Check if the Name field is filled
            EditFieldsList.set_focus(0)
            Yar, Yarr = EditFieldsList.get_focus()
            if (Yar.edit_text.strip() == ''):
                EditFieldsList.set_focus(0)
                self.loop.widget = self.exit
                return
            Information = []
            I = 0
            for I in (0,2,4,6,8,10,12,14,16,18,20,22,24,26,28):
                EditFieldsList.set_focus(I)
                Focus, pos = EditFieldsList.get_focus()
                Information.append(Focus.edit_text)
            #Set original position back
            EditFieldsList.set_focus(Durr)
            Date = datetime.date.today()
            #add the last modification time at the end
            Information.append('%d.%d.%d' % (Date.year, Date.month, Date.day))
            Information = self.ReplaceBlanks(Information)
            if (self.CheckDate(Information[4]) == False):
                EditFieldsList.set_focus(8)
                self.loop.widget = self.WrongDate #aww shizzle! letters in the date or wrong format?
                return
            #Ask if you really reall want to add it :D
            self.loop.widget = self.ReallyAdd
            return
#======================================
    def ViewScreenMenu(self, key):
        '''This menu is for when viewing a record'''
        global INPUT, NumberOfPeople, CameToEdit
        if (self.loop.widget == self.DeletionPopup):
            if key in ('y', 'Y'):
                #If confirmed, delete him and show the next one
                #or the previous one, if deleted person was the last in DB
                database.deleteperson(INPUT[0])
                NumberOfPeople -= 1
                self.loop.widget = self.MainScreen
                if (INPUT[0] == NumberOfPeople):
                    INPUT = database.getperson(INPUT[0]-1)
                    self.MainScreen, self.DeletionPopup = self.ShowInfo()
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.ViewScreenMenu)
                    self.loop.run()
                elif (INPUT[0] < NumberOfPeople):
                    INPUT = database.getperson(INPUT[0]+1)
                    self.MainScreen, self.DeletionPopup = self.ShowInfo()
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.ViewScreenMenu)
                    self.loop.run()
            elif key in ('n', 'N'):
                self.loop.widget = self.MainScreen
                return
        elif key in ('e', 'E'): #Edit screen
            CameToEdit = True
            self.MainScreen, self.exit, self.WrongDate, self.Success, self.ConfirmCancel, self.ReallyAdd = self.ShowEditor()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.CancelAdding)
            self.loop.run()
        elif key in ('a', 'A'):  #Adding screen
            self.MainScreen, self.exit, self.WrongDate ,self.Success, self.ConfirmCancel, self.ReallyAdd = self.ShowAdd()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.CancelAdding)
            self.loop.run()
        elif key in ('d', 'D'): #Deletion of a record popup
            self.loop.widget = self.DeletionPopup
            return
        elif key in ('s', 'S'):  #Search screen
            self.MainScreen, self.Empty, self.GotLetter, self.NoRecord = self.ShowSearch()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.SearchScreenMenu)
            self.loop.run()
        elif key == 'esc':
            self.loop = urwid.MainLoop(self.ShowMain(), Palette, unhandled_input=self.MainScreenMenu)
            self.loop.run()
#======================================
    def SearchResultMenu(self, key):
        '''Menu for the found results'''
        global IntEdit, INPUT, NumberOfPeople
        if (self.loop.widget == self.NoRecord):
            if key == 'esc':
                self.loop.widget = self.MainScreen
                return
        elif key == 'esc':  #Get back to the search screen bish!
            self.loop = urwid.MainLoop(self.ShowMain(), Palette, unhandled_input=self.MainScreenMenu)
            self.loop.run()
        elif key == 'enter':
            if (IntEdit.edit_text == ''): return #nothing is entered...

            if (int(IntEdit.edit_text) > NumberOfPeople) or (int(IntEdit.edit_text) < 1):
                self.loop.widget = self.NoRecord
                return
            
            INPUT = database.getperson(IntEdit.edit_text)
            self.MainScreen, self.DeletionPopup = self.ShowInfo()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.ViewScreenMenu)
            self.loop.run()
        elif key in ('s', 'S'):  #Search
            self.MainScreen, self.Empty, self.GotLetter, self.NoRecord = self.ShowSearch()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.SearchScreenMenu)
            self.loop.run()
        elif key in ('a', 'A'):  #Adding screen
            self.MainScreen, self.exit, self.WrongDate ,self.Success, self.ConfirmCancel, self.ReallyAdd = self.ShowAdd()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.CancelAdding)
            self.loop.run()
#======================================
    def SearchScreenMenu(self, key):
        '''Menu for the search screen'''
        global RadioButtons, INPUT, SearchField
        #Field = ''
        if (self.loop.widget == self.Empty) or (self.loop.widget == self.GotLetter) or (self.loop.widget == self.NoRecord):
            if key == 'esc':
                self.loop.widget = self.MainScreen
                return
        elif key == 'esc':
            self.loop = urwid.MainLoop(self.ShowMain(), Palette, unhandled_input=self.MainScreenMenu)
            self.loop.run()
        elif key == 'enter':
            Hurr = SearchField.edit_text.strip()
            #Lets check if the field is empty, k?
            #Hurr.edit_text = Hurr.edit_text.strip()
            if (Hurr == ''): #oh shi- it is!?
                self.loop.widget = self.Empty #Display an error...
                return
            else:
                #Search by ID starts here
                if (RadioButtons[0].state == True):
                    if (self.CheckForChars(Hurr) == True): #search by ID received a letter!?
                        self.loop.widget = self.GotLetter #show an error
                        return
                    if (int(Hurr) > NumberOfPeople) or (int(Hurr) < 1):
                        self.loop.widget = self.NoRecord
                        return
                    #Get some data from the database
                    INPUT = database.getperson(Hurr)
                    self.MainScreen, self.DeletionPopup = self.ShowInfo()
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.ViewScreenMenu)
                    self.loop.run()

                elif (RadioButtons[1].state == True): #search by name
                    SearchIDs = database.search('name', Hurr)
                    SearchIDs = self.GetResults(SearchIDs)
                    self.MainScreen, self.NoRecord = self.ShowSearchResultScreen(SearchIDs)
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.SearchResultMenu)
                    self.loop.run()

                elif (RadioButtons[2].state == True): #search by last name
                    SearchIDs = database.search('lastname', Hurr)
                    SearchIDs = self.GetResults(SearchIDs)
                    self.MainScreen, self.NoRecord = self.ShowSearchResultScreen(SearchIDs)
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.SearchResultMenu)
                    self.loop.run()

                elif (RadioButtons[3].state == True): #search by nickname
                    SearchIDs = database.search('nickname', Hurr)
                    SearchIDs = self.GetResults(SearchIDs)
                    self.MainScreen, self.NoRecord = self.ShowSearchResultScreen(SearchIDs)
                    self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.SearchResultMenu)
                    self.loop.run()
#======================================
    def MainScreenMenu(self, key):
        '''Main screen menu'''
        if key == 'esc':  #Quit
            raise Exception('\n'*100+'You have quit D0xBase %s. Have a nice day!'.rjust(50) % __version__+'\n'*10)
        elif key in ('s', 'S'):  #Search
            self.MainScreen, self.Empty, self.GotLetter, self.NoRecord = self.ShowSearch()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.SearchScreenMenu)
            self.loop.run()
        elif key in ('a', 'A'):  #Adding screen
            self.MainScreen, self.exit, self.WrongDate ,self.Success, self.ConfirmCancel, self.ReallyAdd = self.ShowAdd()
            self.loop = urwid.MainLoop(self.MainScreen, Palette, unhandled_input=self.CancelAdding)
            self.loop.run()
#======================================
    def Begin(self):
        '''Start everything'''
        self.loop = urwid.MainLoop(self.ShowMain(), Palette, unhandled_input=self.MainScreenMenu)
        self.loop.run()
#======================================

#Check if appropriate version is being used
if urwid.__version__ == '1.0.0': #'0.9.9.2':
    #intialize database
    database =  d0xengine.Db('dox')
    NumberOfPeople = database.connect()
    FrontEnd().Begin()
else:
    print 'Urwid version must be 1.0.0; Exiting.'
